C                                         Last modifications: 26.11.2009
C   *******************************************************************
c              Global description and assumptions:
c              ----------------------------------
c   1. The processes form a 2D nx \times ny logical grid
c   2. Each processor handles arrays of size lx \times ly
c     - lx, ly might be different for the different processes (option)
C   *******************************************************************

      program main

      implicit none
      include 'mpif.h'
c ---------------------------------- MPI stuff ----------------------
c     dimension of the array of processors (nodes):
      integer Comm_2D,status
      integer nPEs,me,ierr,dims(2)
      integer north,south,east,west ! my neighbours in 2D
      integer type_row,type_col,type_brow
      logical periods(2)
      data periods/2*.false./
c ---------------------------------- my stuff ----------------------
      integer nx,ny,nall,lnall,lx,ly,mem_flag,mem_reqR
      integer bufsize,Buffer(5)
      integer O,PVA,POA,PVR,POR,PVB,POB,MEM,
     >          totalV,totalO,Pm,pn,Ps,Pe,Pw,
     >          Pu,Prhs,Pd,Pg,Ph,Pwrk,Pbuf1,Pbuf2
      double precision V
c      structure /Buffer/
c        integer nx
c        integer ny
c        integer lx
c        integer ly
c        integer nall
c        integer lnall
c      end structure

c      parameter (totalV=5625000,totalO=1) ! up to 700 x700
      parameter (totalV=11540500,totalO=1)  ! up to 1024 x 1024
      DIMENSION  V(totalV),O(totalO)

      COMMON/CRNTADD/ PVA,POA,PVR,POR,PVB,POB
      COMMON/type_vect/ type_row,type_col,type_brow
       
      call MPI_INIT(ierr)
c      write(*,*) '>>>>>>',ierr
      call MPI_COMM_RANK( MPI_COMM_WORLD, me,    ierr)
      call MPI_COMM_SIZE( MPI_COMM_WORLD, nPEs,ierr)
      if (me.EQ.0) then
        write(*,*) 'The number of processors (nPEs) is: ',nPEs
      endif

c -------------------------------------------------------------------
c Let MPI find a "good" decomposition.
c Get a new communicator for a decomposition of the domain.
c
c  dims  contains number of PEs in each coordinate direction
c  Comm_2D  is the corresponding communicator

      dims(1) = 0
      dims(2) = 0
      call MPI_DIMS_CREATE( nPEs, 2, dims, ierr )
      call MPI_CART_CREATE( MPI_COMM_WORLD, 2, dims, periods, .true.,
     >                    Comm_2D, ierr )
c
c Get my position in this communicator
c
      call MPI_COMM_RANK( Comm_2D, me, ierr )
      write(*,*) 'Process ', me, ' of ', nPEs, ' is alive.'
c
c -------------------------- My neighbors are now +/- 1 with my rank.
      call fnd2dnbrs( Comm_2D, west, east, north, south )
      write(*,*) 'Process ', me, ':',north,south,east,west
      
      nx = dims(1) ! number of processes in 'x'-direction
      ny = dims(2) ! number of processes in 'y'-direction

      if (me.eq.0) then
c	 lx = 64
c	 ly = 64
c       write(*,*) 'Enter number of points in x-direction (per subgrid):'
       read(1,*) lx ! number of 'x'-points per subgrid
c       write(*,*) 'Enter number of points in y-direction (per subgrid):'
       read(1,*) ly ! number of 'y'-points per subgrid

         lnall = lx*ly        ! subgridsize
         nall  = nx*ny*lnall  ! total number of unknowns
         bufsize = max(lx,ly) ! 
c ---------- memory requirements test:
      mem_reqR = 10*lnall + (lx+2)*(ly+2) + 2*bufsize
      mem_flag = 0

      write(*,*) 'Real*8 memory required: ',
     >            mem_reqR,'/',totalV
      if (mem_reqR .gt. totalV) then
          write(*,*) 'Real memory not enough.'
          mem_flag = 1
      endif
      if (mem_flag .eq. 1) goto 99999

c ---------- prepare for broadcasting the initial parameters
         Buffer(1) = lx
         Buffer(2) = ly
         Buffer(3) = nall
         Buffer(4) = lnall
         Buffer(5) = bufsize
      endif
         call MPI_BCAST(Buffer,5,MPI_INTEGER, 
     >                  0 ,MPI_COMM_WORLD, ierr)

      call MPI_BARRIER(MPI_COMM_WORLD,ierr)
c      write(*,*) 'Bcasted:','(',me,')',Buffer
      if (me.ne.0) then
         lx    = Buffer(1)
         ly    = Buffer(2)
         nall  = Buffer(3)
         lnall = Buffer(4)
	 bufsize = Buffer(5)
      endif
      
      
c --------------------------------------------------------- Make comm_types:
c ---------------------------------------------------------  take a row:
          CALL MPI_Type_vector ( ly, 1, lx,
     >                           MPI_DOUBLE_PRECISION,
     >                           type_row, ierr )
          if ( ierr .NE. MPI_SUCCESS ) then
            stop
          end if
          CALL MPI_Type_commit ( type_row, ierr )
          if ( ierr .NE. MPI_SUCCESS ) then
            stop
          endif
c ---------------------------------------------------------  take a column:
          CALL MPI_Type_vector ( lx, 1, 1,
     >                           MPI_DOUBLE_PRECISION,
     >                           type_col, ierr )
          if ( ierr .NE. MPI_SUCCESS ) then
            stop
          end if
          CALL MPI_Type_commit ( type_col, ierr )
          if ( ierr .NE. MPI_SUCCESS ) then
            stop
          endif
c ---------------------------------------------------------  take row (buf):
          CALL MPI_Type_vector ( ly, 1, lx+2,
     >                           MPI_DOUBLE_PRECISION,
     >                           type_brow, ierr )
          if ( ierr .NE. MPI_SUCCESS ) then
            stop
          end if
          CALL MPI_Type_commit ( type_brow, ierr )
          if ( ierr .NE. MPI_SUCCESS ) then
            stop
          endif
      
c ---------- memory allocation
      CALL RESET(1,     'VA')
      CALL RESET(1,     'OA')
      CALL RESET(totalV,'VB')
      CALL RESET(totalO,'OB')

c ---------- for the matrix
      Pm = MEM(lnall,'VA')
      Pn = MEM(lnall,'VA')
      Ps = MEM(lnall,'VA')
      Pe = MEM(lnall,'VA')
      Pw = MEM(lnall,'VA')
c ---------- for the vectors
      Pu   = MEM(lnall,        'VA')
      Prhs = MEM(lnall,        'VA')
      Pd   = MEM(lnall,        'VA')
      Pg   = MEM(lnall,        'VA')
      Ph   = MEM(lnall,        'VA')
      Pwrk = MEM((lx+2)*(ly+2),'VA')
      Pbuf1= MEM(bufsize,      'VA')
      Pbuf2= MEM(bufsize,      'VA')
      
            call main_main(V(Pm),V(Pn),V(Ps),V(Pe),V(Pw),
     >     V(Pu),V(Prhs),V(Pd),V(Pg),V(ph),
     >     V(Pwrk),V(Pbuf1),V(Pbuf2),
     >     north,south,east,west,           
     >     me, nPEs,nx,ny,lx,ly,nall,lnall,bufsize)

99999 call MPI_FINALIZE( ierr )
      stop
      end

c -------------------------------------------------------------------
      subroutine main_main(m,n,s,e,w,
     >     u,rhs,D,G,H,wrk,buf1,buf2,
     >     north,south,east,west,
     >     me, nPEs,nx,ny,lx,ly,nall,lnall,bufsize)
      implicit none
      include 'mpif.h'
c ---------------------------------- MPI stuff ----------------------
c     dimension of the array of processors (nodes):
      integer nPEs,me,iter,res,floor,ierr
      integer north,south,east,west
      double precision start_time,end_time !,MPI_WTIME
c ---------------------------------- my stuff -----------------------
      integer nx,ny,nall,lnall,lx,ly,bufsize
      double precision eps,Ceps,arfac
      double precision m(lx,ly),
     >                 n(lx,ly),s(lx,ly),
     >                 e(lx,ly),w(lx,ly),
     >                 u(lx,ly),rhs(lx,ly),
     >                 D(lx,ly),G(lx,ly),
     >                 H(lx,ly),wrk(0:lx+1,0:ly+1),
     >                 buf1(bufsize),buf2(bufsize)

c ---------- find the neighbours
c      write(*,*) 'The very beginning:','(',me,')',nx,ny

      call matgen(m,n,s,e,w,lx,ly,me,north,south,east,west)
c      write(*,*) '<><><> After matgen','(',me,')'
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)

      call rhsgen(rhs,u, m,n,s,e,w,
     >            wrk, buf1, buf2,
     >            me,north,south,east,west,
     >            lx, ly, bufsize)
c      write(*,*) '<><><> After rhsgen','(',me,')'
      eps = 1.0d-12
      start_time = MPI_WTIME()
      call mpi_cg(u, rhs, m,n,s,e,w,
     >           D, G, H, wrk, buf1, buf2,
     >           me,north,south,east,west,
     >           eps, Ceps, iter, arfac, lx, ly, bufsize)
      end_time = MPI_WTIME()
      write(*,*) 'Wall clock time for: ',me,': ',end_time-start_time
      if (me.eq.0) then
          write(*,*) 'Iterations: ',iter,',Av. reduction factor: ',arfac 
      endif
999   return
      end
c------------------------------------------------------------------------
c fnd2dnbrs: determine the neighbors in a 2-d decomposition of the domain,
c            assuming that MPI_Cart_create has already been called.
c------------------------------------------------------------------------
      subroutine fnd2dnbrs( Comm_2D,
     >                      nbrleft, nbrright, nbrtop, nbrbottom )
      implicit none
      integer Comm_2D, nbrleft, nbrright, nbrtop, nbrbottom
      integer ierr

      call MPI_Cart_shift( Comm_2D, 0,  1, nbrleft,   nbrright, ierr )
      call MPI_Cart_shift( Comm_2D, 1,  1, nbrbottom, nbrtop,   ierr )

      return
      end
c------------------------------------------------------------------------
